#ifndef NachrichtenListe_H
#define NachrichtenListe_H

#include <Klient.h>
#include <Array.h>
#include <Fenster.h>
#include <Knopf.h>
#include <GSLSoundV.h>
#include <functional>
#include <Fortschritt.h>
#include "../Update/Update.h"
#include "..\..\Aktionen\AktionsThread.h"
#include <Rahmen.h>

using namespace Framework;
using namespace Network;

class NLNachricht
{
protected:
    TextFeld *titel;
    Knopf *close;
    LRahmen *rahmen;
    int hhe;
    int maxHhe;
    bool ausgewhlt;
    int animation;
    double tickVal;
    bool rend;
    int ref;

    NLNachricht( Schrift *zSchrift, Text *titel );
    virtual ~NLNachricht();
public:
    virtual void entfernen();
    void setAusgewhlt( bool ausgewhlt );
    virtual void doMausEreignis( MausEreignis &me );
    virtual bool tick( double tickVal );
    virtual void render( int y, Bild &zRObj );
    // contant
    bool istAusgewhlt() const;
    int getHeight() const;
    Text *zTitel() const;
    // Reference Counting
    NLNachricht *getThis();
    NLNachricht *release();
};

class SpielUpdateNachricht : public NLNachricht
{
private:
    TextFeld *text;
    Knopf *herunterladen;
    Knopf *abbrechen;
    std::function<void()> posAk;
    std::function<void()> negAk;
    FBalken *fortschritt;
    Update *update;

public:
    SpielUpdateNachricht( Schrift *zSchrift, Text *titel, Text *nachricht, int dgID, std::function<void()> posAk = []() {}, std::function<void()> negAk = []() {} );
    ~SpielUpdateNachricht();
    // nicht constant
    void entfernen() override;
    void doMausEreignis( MausEreignis &me ) override;
    bool tick( double tickVal ) override;
    void render( int y, Bild &zRObj ) override;
};

class Nachricht : public NLNachricht
{
private:
	TextFeld *text;
	Knopf *positiv;
	Knopf *negativ;
    std::function<void()> posAk;
    std::function<void()> negAk;

public:
	// Konstruktor
    Nachricht( Schrift *zSchrift, Text *titel, Text *nachricht, Text *positiv, Text *negativ, std::function<void()> posAk = []() {}, std::function<void()> negAk = []() {} );
	// Destruktor
	~Nachricht();
	// nicht constant
    void entfernen() override;
	void doMausEreignis( MausEreignis &me ) override;
	bool tick( double tickVal ) override;
	void render( int y, Bild &zRObj ) override;
};

class NachrichtenListeObj : public Zeichnung
{
private:
	RCArray< NLNachricht > *members;
	VScrollBar *vsb;
	Schrift *schrift;
	Punkt bildschirmGre;
	int anzahl;

public:
	// Konstruktor
	NachrichtenListeObj( Schrift *zSchrift );
	// Destruktor
	~NachrichtenListeObj();
	// nicht constant
	void addNachricht( NLNachricht *nachricht );
	void removeNachricht( NLNachricht *zNachricht );
	void removeAll();
	bool tick( double tickVal ) override;
	void doMausEreignis( MausEreignis &me ) override;
	void render( Bild &zRObj ) override;
};

class NachrichtenListe
{
private:
    Schrift *schrift;
	Fenster *fenster;
	Knopf *minMax;
	NachrichtenListeObj *nachrichtenListe;
	Punkt bildschirmGre;
	TextFeld *berschrift;
	Bild *minimierenBild;
	Bild *maximierenBild;
	GSL::GSLSoundV *msgSound;
	GSL::GSLSoundV *errSound;
	double tickVal;
	int animation;
	bool rend;
	int ref;

public:
	// Konstruktor
	NachrichtenListe( Schrift *zSchrift, Fenster *zNachLoginFenster );
	// Destruktor
	~NachrichtenListe();
	// nicht constant
	void addNachricht( Text *titel, Text *nachricht, Text *positiv = 0, Text *negativ = 0, std::function<void()> posAk = []() {}, std::function<void()> negAk = []() {} );
    void addNachricht( NLNachricht *nachricht );
	void removeNachricht( NLNachricht *zNachricht );
	void removeAll();
	bool druckMinMax( MausEreignis &me );
	bool tick( double tickVal );
	// contant
    Schrift *zSchrift() const;
	// Reference Counting
	NachrichtenListe *getThis();
	NachrichtenListe *release();
};

// Nachrichten
bool NachrichtenListeMinMaxME( void *p, void *obj, MausEreignis me );

#endif