#ifndef KESSVorschau_H
#define KESSVorschau_H

#include <KSGScript.h>
#include <AuswahlBox.h>
#include <Thread.h>
#include <Rahmen.h>

using namespace Framework;
using namespace KSGScript;

class KESSVorschauKarteScript : public Thread
{
private:
    KSGScriptObj *fenster;
    Punkt pos;
    LRahmen *ram;
    HINSTANCE ksgs;
    Text pfad;
    bool sichtbar;
    bool besitztTestVersion;
    int verbleibend;
    bool vollversionErwerbbar;
    bool testversionErwerbbar;
    int vvPreis;
    int tvPreis;
    int kupfer;
    unsigned char alpha;
    bool aktion;
    int ak;

public:
    // Konstruktor
    KESSVorschauKarteScript( Schrift *zSchrift, TextFeld *zLog );
    // Destruktor
    ~KESSVorschauKarteScript();
    // nicht constant
    void thread() override;
    void setScriptParams( bool hatTV, int tvVerb, bool vvEn, bool tvEn, int vvK, int tvK, int k );
    void ladeKarteSeite( char *pfad );
    void ksgsAktion( RCArray< KSGSVariable > *parameter, KSGSVariable **retVal );
    void doMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    bool tick( double zeit );
    void render( Bild &zRObj );
};

class KESSVorschauKarte
{
private:
    Knopf *auswhlen;
    Bild *hintergrund;
    AlphaFeld *ausgewhlt;
    LRahmen *ram;
    bool ausw;
    bool rend;
    int ref;

public:
    // Konstruktor
    KESSVorschauKarte( Schrift *schrift );
    // Destruktor
    ~KESSVorschauKarte();
    // nicht constant
    void ladeKarte( char *pfad );
    bool doMausEreignis( MausEreignis &me );
    bool tick( double zeit );
    void render( Bild &zRObj );
    // Reference Counting
    KESSVorschauKarte *getThis();
    KESSVorschauKarte *release();
};

class KESSVorschau : public Zeichnung
{
private:
    Schrift *schrift;
    TextRenderer *tr;
    KESSVorschauKarteScript *script;
    KESSVorschauKarte *karte;
    Knopf *beenden;
    AuswahlBox *besitztTestVersion;
    TextFeld *spieleVerbleibend;
    AuswahlBox *vvErwerbbar;
    AuswahlBox *tvErwerbbar;
    TextFeld *vvPreis;
    TextFeld *tvPreis;
    TextFeld *kupfer;
    Text pfad;
    TextFeld *log;
    unsigned char alpha;
    bool sichtbar;
    double tickVal;
    int aktion;
    bool rend;

public:
    // Konstruktor
    KESSVorschau( Schrift *zSchrift );
    // Destruktor
    ~KESSVorschau();
    // nicht constant
    int getAktion();
    void ladeKarte( char *pfad );
    void setSichtbar( bool sicht );
    void doMausEreignis( MausEreignis &me );
    void doTastaturEreignis( TastaturEreignis &te );
    bool tick( double zeit );
    void render( Bild &zRObj );
    // constant
    bool istSichtbar() const;
};

#endif