#ifndef KartenKaufen_H
#define KartenKaufen_H

#include <Schrift.h>
#include <Knopf.h>
#include <KSGScript.h>
#include <Array.h>
#include <Scroll.h>
#include <Animation.h>
#include <Thread.h>
#include <AuswahlBox.h>
#include <Rahmen.h>

using namespace Framework;
using namespace KSGScript;

class KartenKaufenAuswahl : public Thread
{
private:
	int karteId;
	unsigned char alpha;
	KSGScriptObj *fenster;
	Framework::Animation2D *laden;
	Punkt pos;
	LRahmen *ram;
	HINSTANCE ksgs;
	bool aktion;

public:
	// Konstruktor
	KartenKaufenAuswahl( Schrift *zSchrift );
	// Destruktor
	~KartenKaufenAuswahl();
	// nicht constant
	void ladeKarteSeite( int id );
	void reset();
	virtual void thread();
	void ksgsAktion( RCArray< KSGSVariable > *parameter, KSGSVariable **retVal );
	void doMausEreignis( MausEreignis &me );
	void doTastaturEreignis( TastaturEreignis &te );
	bool tick( double zeit );
	void render( Bild &zRObj );
};

class KartenKaufenListeEintrag
{
private:
	int karteId;
	Knopf *auswhlen;
	Bild *hintergrund;
	AlphaFeld *ausgewhlt;
	LRahmen *ram;
	bool ausw;
	bool rend;
	int ref;

public:
	// Konstruktor
	KartenKaufenListeEintrag( int id, Schrift *schrift );
	// Destruktor
	~KartenKaufenListeEintrag();
	// nicht constant
	void resetAuswahl();
	bool doMausEreignis( MausEreignis &me );
	bool tick( double zeit );
	void render( int yOff, Bild &zRObj );
	// constant
	int getKarteId() const;
	// Reference Counting
	KartenKaufenListeEintrag *getThis();
	KartenKaufenListeEintrag *release();
};

class KartenKaufenListe
{
private:
	Schrift *s;
	LRahmen *ram;
	Punkt pos;
	RCArray< KartenKaufenListeEintrag > *eintrge;
	VScrollBar *vScroll;
	bool rend;
	int ref;
	Critical cs;

public:
	// Konstruktor
	KartenKaufenListe( Schrift *schrift );
	// Destruktor
	~KartenKaufenListe();
	// nicht constant
	void ladeKarten( Array< int > *karten );
	void leeren();
	int doMausEreignis( MausEreignis &me );
	bool tick( double zeit );
	void render( Bild &zRObj );
	// constant
	int getEintragAnzahl();
	// Reference Counting
	KartenKaufenListe *getThis();
	KartenKaufenListe *release();
};

class KartenKaufen : public Thread
{
private:
	unsigned char alpha;
	bool sichtbar;
	Text *suchText;
	LRahmen *ram;
	AuswahlBox *spielArt;
	TextFeld *suchFilterT;
	TextFeld *suchFilter;
	Knopf *suchen;
	TextFeld *seiten;
	Knopf *weiter;
	Knopf *zurck;
	Punkt pos;
	KartenKaufenListe *liste;
	KartenKaufenAuswahl *auswahl;
	Framework::Animation2D *laden;
	double tickVal;
	int seite;
	int maxSeite;
	int aktion;
	bool rend;

public:
	// Konstruktor
	KartenKaufen( Schrift *zSchrift );
	// Destruktor
	~KartenKaufen();
	// nicht constant
	void setSichtbar( bool sicht );
	virtual void thread();
	void doMausEreignis( MausEreignis &me );
	void doTastaturEreignis( TastaturEreignis &te );
	bool tick( double zeit );
	void render( Bild &zRObj );
	// constant
	bool istSichtbar() const;
};

#endif